# Параметризация

Параметризация позволяет тестировать производительность кода с различными входными данными.

## Конфигурирование

### Аннотация

1. Параметры для полей класса
```bsl
&Параметры("MD5", "SHA256")
Перем АлгоритмХеширования Экспорт;
```

2. Параметры для методов бенчмарка
```bsl
// Каждый &Параметры определяет новый тестовый случай
&Параметры(50, 100) // Парам1=50, Парам2=100
&Параметры(75, 150) // Парам1=75, Парам2=150
Процедура Бенчмарк(Парам1, Парам2) Экспорт
```

### API

1. Параметры для полей класса
```bsl
// Добавление значений для конкретного поля
Конфигурация
	.ДобавитьПараметр("АлгоритмХеширования", "MD5")
	.ДобавитьПараметр("АлгоритмХеширования", "SHA256");
```

2. Параметры для методов бенчмарка
```bsl
// Создание набора параметров для метода
НаборПараметров = Новый Массив();
НаборПараметров.Добавить(100);
НаборПараметров.Добавить(200);

// Регистрация набора
ДескрипторБенчмарка.ДобавитьПараметры(НаборПараметров);
```

## Пример

```bsl
&Параметры(5, 0)
Перем ДобавляемаяЗадержка Экспорт; // Число

&Бенчмарк
&Параметры(100, 10)
&Параметры(100, 20)
&Параметры(200, 10)
&Параметры(200, 20)
Процедура Бенчмарк(Задержка1, Задержка2) Экспорт
	Приостановить(Задержка1 + Задержка2 + ДобавляемаяЗадержка);
КонецПроцедуры
```

### Результаты выполнения

| Method   | ДобавляемаяЗадержка | Задержка1 | Задержка2 |     Mean |  StdErr |    StdDev |   Median |  Op/s |
|----------|--------------------:|----------:|----------:|---------:|--------:|----------:|---------:|------:|
| Бенчмарк |                   0 |       100 |        10 | 110.8 ms | 39.2 us | 151.77 us | 110.9 ms | 9.022 |
| Бенчмарк |                   0 |       100 |        20 | 120.9 ms | 32.3 us | 124.97 us | 120.9 ms | 8.273 |
| Бенчмарк |                   0 |       200 |        10 | 210.9 ms | 30.2 us | 116.98 us | 210.9 ms | 4.742 |
| Бенчмарк |                   0 |       200 |        20 | 220.9 ms | 32.9 us | 127.50 us | 220.9 ms | 4.527 |
| Бенчмарк |                   5 |       100 |        10 | 115.9 ms | 19.0 us |  73.58 us | 115.9 ms | 8.627 |
| Бенчмарк |                   5 |       100 |        20 | 125.8 ms | 29.4 us | 114.00 us | 125.8 ms | 7.946 |
| Бенчмарк |                   5 |       200 |        10 | 215.9 ms | 26.6 us | 103.01 us | 215.9 ms | 4.632 |
| Бенчмарк |                   5 |       200 |        20 | 225.9 ms | 31.2 us | 121.02 us | 226.0 ms | 4.427 |
