# Среда выполнения

Для сравнения производительности в разных версиях **OneScript** используется аннотация `&ИсполняющаяСреда`, позволяющая запускать бенчмарки в нескольких средах одновременно.

## Конфигурирование

### Аннотации

1. Базовый вариант (несколько версий через запятую)
```bsl
&ИсполняющаяСреда("stable, dev, 1.9.3")
Процедура ПриСозданииОбъекта()
```

2. С кастомным именем для отчётов
```bsl
&ИсполняющаяСреда(Версия = "1.9.3", Наименование = "Стабильная сборка")
&ИсполняющаяСреда(Версия = "dev", Наименование = "Ночная сборка")
Процедура ПриСозданииОбъекта()
```

3. Указание эталонной версии для сравнения
```bsl
&ИсполняющаяСреда(
    Версия = "stable",
    Наименование = "Стабильная сборка",
    ЭтоЭталон = Истина
)
Процедура ПриСозданииОбъекта()
```

4. Использование кастомного исполняемого файла
```bsl
&ИсполняющаяСреда(Версия = "path/to/oscript.exe", Наименование = "Моя сборка")
Процедура ПриСозданииОбъекта()
```

### API

1. Простой вариант указания версий
```bsl
Конфигурация
    .ДобавитьВерсиюИсполняющейСреды("stable")
    .ДобавитьВерсиюИсполняющейСреды("dev")
    .ДобавитьВерсиюИсполняющейСреды("1.9.3");
```

2. С кастомным именем для отчётов
```bsl
Конфигурация
    .ДобавитьВерсиюИсполняющейСреды("1.9.3", "Стабильная сборка")
    .ДобавитьВерсиюИсполняющейСреды("dev", "Ночная сборка");
```

3. Указание эталонной версии
```bsl
Конфигурация.ДобавитьВерсиюИсполняющейСреды("stable", "Стабильная сборка", Истина);
```

4. Путь к исполняющему файлу
```bsl
Конфигурация.ДобавитьВерсиюИсполняющейСреды("path/to/oscript.exe", "Моя сборка");
```

### CLI

Для указания версий OneScript используйте опцию `--runtime` со значениями через запятую:

```bash
benchos run --runtime=stable,dev,1.9.3 [FILE]
```

Подробнее см. в разделе [CLI](CLI.md).

## Пример

```bsl
Перем ДвоичныеДанные;

&ИсполняющаяСреда("dev")
&ИсполняющаяСреда(
    Версия = "stable", 
    Наименование = "Стабильная", 
    ЭтоЭталон = Истина
)
Процедура ПриСозданииОбъекта()
КонецПроцедуры

&ПередВсеми
Процедура Инициализация(Контекст) Экспорт
	ДвоичныеДанные = ПолучитьДвоичныеДанныеИзСтроки("Много данных");
КонецПроцедуры

&Бенчмарк
Процедура SHA256() Экспорт
	ХешированиеДанных = Новый ХешированиеДанных(ХешФункция.SHA256);
	ХешированиеДанных.Добавить(ДвоичныеДанные);
	ХешСумма = ХешированиеДанных.ХешСумма;
КонецПроцедуры
```

### Результаты выполнения

| Method | Runtime               | Baseline |      Mean |   StdErr |   StdDev | Ratio | RatioSD |    Op/s |
|--------|-----------------------|----------|----------:|---------:|---------:|------:|--------:|--------:|
| SHA256 | 2.0.0-rc.8+608        | No       |  9.048 us | 93.81 ns | 363.3 ns |  0.67 |    0.03 | 110,523 |
| SHA256 | Стабильная (1.9.3.15) | Yes      | 13.561 us | 70.97 ns | 274.9 ns |  1.00 |    0.00 |  73,742 |
