# Управление итерациями и вызовами

## Количество итераций

Контролирует количество измерительных итераций для основных этапов (измерение и память).

### Конфигурирование

#### Аннотация

```bsl
&КоличествоИтераций(25)
Процедура ПриСозданииОбъекта()
```

#### API

```bsl
Конфигурация.УстановитьКоличествоИтераций(25);
```

#### CLI

```bash
benchos run --iterationCount 25 [FILE]
```


## Количество вызовов за итерацию

Указывает, сколько раз метод бенчмарка должен быть вызван в течение одной итерации. 

Особенности:
- Если установлено `0`, то количество будет рассчитано автоматически.
- Игнорируется для стратегии [`ХолодныйЗапуск`](СредаВыполнения.md#холодныйзапуск).

### Конфигурирование

#### Аннотация

```bsl
&КоличествоВызововЗаИтерацию(1000)
Процедура ПриСозданииОбъекта()
```

#### API

```bsl
Конфигурация.УстановитьКоличествоВызововЗаИтерацию(1000);
```

#### CLI

```bash
benchos run --invocationCount 1000 [FILE]
```


## Количество прогревочных итераций

Указывает количество прогревочных итераций для подготовки окружения перед выполнением основных замеров бенчмарка. Прогревочные итерации используются для стабилизации системы: прогрев кэшей, 
инициализация внешних ресурсов. Результаты этих итераций не учитываются в финальной статистике. 

Особенности:
- Игнорируется для стратегии [`ХолодныйЗапуск`](СредаВыполнения.md#холодныйзапуск).

### Конфигурирование

#### Аннотация

```bsl
&КоличествоПрогревочныхИтераций(10)
Процедура ПриСозданииОбъекта()
```

#### API

```bsl
Конфигурация.УстановитьКоличествоПрогревочныхИтераций(10);
```

#### CLI

```bash
benchos run --warmupCount 10 [FILE]
```


## Минимальное время итерации

Указывает минимальное время выполнения одной итерации бенчмарка в миллисекундах. Используется для подбора количества вызовов за итерацию так, чтобы одна итерация длилась не меньше указанного времени.

Особенности:
- Игнорируется для стратегии [`ХолодныйЗапуск`](СредаВыполнения.md#холодныйзапуск).

### Конфигурирование

#### Аннотация

```bsl
&МинимальноеВремяИтерации(250)
Процедура ПриСозданииОбъекта()
```

#### API

```bsl
Конфигурация.УстановитьМинимальноеВремяИтерации(250);
```

#### CLI

```bash
benchos run --iterationTime 250 [FILE]
```


## Минимальное количество вызовов за итерацию

Гарантированное мминимальное количество вызовов метода в течение одной итерации бенчмарка.

Особенности:
- Игнорируется для стратегии [`ХолодныйЗапуск`](СредаВыполнения.md#холодныйзапуск).

### Конфигурирование

#### Аннотация

```bsl
&МинимальноеКоличествоВызововЗаИтерацию(8)
Процедура ПриСозданииОбъекта()
```

#### API

```bsl
Конфигурация.УстановитьМинимальноеКоличествоВызововЗаИтерацию(8);
```